function [tau2,varargout]=find_tau2(P,F,Opt)
%%  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
% === A supplementary code for "Characterizing MSRE Models" 
% === Written by Seonghoon Cho, October 9, 2011
% === For a model with P, and F, this code searches for tau2 such that 
% === tau2 = min (R_BarPsi_LkL) with Lambda subject to the restriction in 
% === Proposition 2 and the argmin, Lambda(s(t),s(t+1)).
% ==================================================================
% Some Definitions first
%   Model : w(t)=E_t [F(s(t),s(t+1))w(t+1)]
%   Solution : w(t+1)=Lambda(s(t),s(t+1))w(t)+V(s(t+1))V(s(t+1)'*eta(t+1) 
%       where  Lambda(s(t),s(t+1))=V(s(t+1))*Phi(s(t),s(t+1))*V(s(t)).                     
%   Matrices,  Psi_X and BarPsi_X where X can be
%            L (Lambda), LkL for kron(L,L)
%            F, FkF for kron(F,F), 
%            FkLt for kron(F,L^T) with T being "transpose" 
%   Xij being X(s(t)=i,s(t+1)=j)
%   R_Psi_X is the spectral radius of Psi_X
%      
%% This code does the following.
%  (1) For given P and F, computes tau2=min (R_BarPsi_LkL) 
%       specificed in find_tau2Q.m subject to
%           V{i}^T V{i} =eye(k{i}), sum_j P(i,j)F{i,j}V{j}Psi{i,j} =V{i} 
%       specified in tau2NL.m, and Lambdamin=argmin (R_BarPsi_LkL).
%  Remark 1. Confirm Lemma 4 : tau2 >= (1/xi2) where xi2 = R_Psi_FkF, and 
%      conjecture tau2 = 1/xi2 always (if the model has no absoring state.)
%  Remark 2: This codes will produce different Lambdamin whenever executed
%           because it starts with a randomized initial vaules, but will 
%           confirm that tau >= 1/xi2. The problem is like to find (x,y) to 
%           minimize z=(x+y-1)^2. Thus, different initial choice of 
%           (x0, y0) will yield the same z=0 with different argmin of z.
%% Contents of the Code
%   [1] Specify your own Input and run the code.
%   [2] Default Initial values for ks, V and Phi 
%   [3] Optimization
%   [4] Output
%% Input and Output
%   [1] Input arguments
%       [1-1] Required Input arguments
%           P(1,1)=p11;.... P(S,S)=pSS;
%                  : S by S transition matrix where the ij-th element, 
%                  p_ij=Pr(s(t+1)=j|s(t)=i).
%           F=cell(S,S); F{1,1}=Fij; ..., F{S,S}=FSS;
%                  : S by S cell array such that F{i,j}=F(s(t)=i,s(t+1)=j).
%                  (If F=F(s(t)), then A is S by 1 cell array such that
%                    F{i,1}=F(s(t)=i).) size(F{i,j})=n by n
%       [1-2] Optional Input arguments
%           Opt.ks  : S by 1 vector of [k(s(t)=1) ... k(s(t)=S)]' 
%                 where 0<= k(i) <= n for all i and
%                       0<  k(i) for some i
%                 - Default : ks=[n_1 ... n_S]' where n_i=rank(F{i,j))                   
%           Opt.V   : S by 1 cell array such that V{i} is the  n by k(i) 
%                 initial matrix with orthornormal columns
%                 - Default : columns of V{i) are random orthonormal.
%           Opt.Phi : S by S cell array such that Phi{i,j} is the  
%                 k(j) by k(i) initial matrix.
%                 - Default : Phi{i,j} is random. 
%   [2] Output arguments
%           tau2    : min (R_BarPsi_LkL)
%           Lambda  : argmin (R_BarPsi_LkL)
%           V         : V{j} corresponding to Lambda
%           OtherOutput : Structure of Other output information 
%               OtherOutput.Psi : Psi{i,J} corresponding to Lambda
%               OtherOutput.R_BarPsi_L : BarPsi_Lambda
%               OtherOutput.R_Psi_FkF : R_Psi_kron(F,F)
%               OtherOutput.R_Psi_F   : R_Psi_kron(F)
%               OtherOutput. Psi_LtkF  : Psi_kron(Lambda',F)


%% [1] Identify S (number of Regime), n (rows of A) 
    S=size(P,1); 
    n=size(F{1,1},1);   

%% [2] Default Specifications for ks, V and Phi

    if nargin==3 && isfield(Opt,'ks'), ks=Opt.ks; 
    else ks=zeros(S,1);                                        
        for i=1:S,
            rankj=zeros(S,1); 
                for j=1:S, rankj(j)=rank(F{i,j}); end
                ks(i)=max(rankj);
        end
	end
    if nargin==3 && isfield(Opt,'V'), V=Opt.V; 
    else k=cell(S,1); W=cell(S,1); V=cell(S,1);     
        for j=1:S,
            k{j,1}=ks(j,1);
            W{j,1}=orth(randn(n,n)); % Random orthonormal basis
            V{j,1}=W{j,1}(:,1:k{j,1});
        end
	end
    if nargin==3 && isfield(Opt,'Phi'), Phi=Opt.Phi; 
    else Phi=cell(S,S);
        for i=1:S, for j=1:S,
            Phi{i,j}=(randn(k{j},k{i})); 
        end, end      
	end
          
%% [3] Constrained Optimization
%     Search over V{i}, Phi{i,j} subject to Vi'Vi=eye(ki) and 
%     Sum[p_ij F_ij L_ij]=Vi Vi',    

    %% (3-1) Compute Psi_F, Psi_FkF,  and xi=R_Psi_FkF   
        Psi_F=[]; Psi_FkF=[]; 
        for i=1:S
            tmp1=[]; tmp2=[];
                for j=1:S, 
                    tmp1=[tmp1 P(i,j)*F{i,j}]; 
                    tmp2=[tmp2 P(i,j)*kron(F{i,j},F{i,j})]; 
                end
            Psi_F=[Psi_F;tmp1]; Psi_FkF=[Psi_FkF;tmp2];
        end
        R_Psi_F=max(abs(eig(Psi_F))); R_Psi_FkF=max(abs(eig(Psi_FkF)));
        xi1=R_Psi_F;
        xi2=R_Psi_FkF; 
    %% (3-2) vectorize the parameters
        vecPhi=[];  for i=1:S, 
                        for j=1:S, vecPhi=[vecPhi;Phi{i,j}(:)]; end, 
                    end
        vecV=[]; for i=1:S, vecV=[vecV;V{i}(:)]; end
        par0=[vecPhi;vecV]; % initial parameter vector.
    
    %% (3-3) Index matrices  KKSbegin and KKSend for identifying 
    %%      Psi{i,j} from vecPhi
    %%      vec(Phi{i,j}) = par0 ( KKSbegin(i,j):KKSend(i,j) )
        KK=[]; 
        for i=1:S, for j=1:S,
            KK(i,j)=k{i}*k{j}; % dimension of vec(Phi{i,j})
        end; end
            KKT=KK';
        tmp=KKT(:);
        KKSendv=[];
        for i=1:S^2, KKSendv(i)=sum(tmp(1:i)); end
            
        KKSend=reshape(KKSendv,S,S)';  KKSendT=KKSend';
            vKKSendt=KKSendT(:);
            vKKSbegint=[1;vKKSendt(1:end-1)+1];
        KKSbegin=reshape(vKKSbegint,S,S)';
        
    %% (3-4) Optimization 
         options=optimset('Algorithm','interior-point','Display','iter',...
             'MaxFunEvals',200000,'MaxIter',2000,'DerivativeCheck','on',...
             'LargeScale','on','TolFun',1e-8,'TolX',1e-8,'Diagnostics',...
             'off');
         [par,fval,exitflag] = fmincon(@(par) ...
             find_tau2Q(par,n,S,k,KKSbegin,KKSend,P),...
             par0,[],[],[],[],[],[],@(par)...
             find_tau2NL(par,n,S,k,KKSbegin,KKSend,P,F),options);      
         [R_BarPsi_LkL,L,Phi,V,BarPsi_LkL]=...
             find_tau2Q(par,n,S,k,KKSbegin,KKSend,P); % Output
         
         tau2=R_BarPsi_LkL;
         Lambda=L;    
    varargout{1}=Lambda;
    varargout{2}=V;
%% [4] OutPut       
        disp('Let xi2 =R_Psi_FkF and tau2 = R_BarPsi_LkL')
        disp('Check whether tau2>=1/xi2')
        disp('[xi2 tau2 1/xi2] are')
        disp([xi2 tau2 1/xi2])        
        
    Abs_state=[];
    for s=1:S, if P(s,s)==1, Abs_state=[Abs_state s]; end, end
    if ~isempty(Abs_state) && tau2 > 1/xi2+0.001, 
        disp('===================================')
        disp('The following states are absorbing')
        disp(Abs_state)
        disp('  If tau2 > 1/xi2, then one may need to run this code with')
        disp('  k(i) from 0 to maximum rank of F(i,j) for all i.')
        disp('  Example: If 2 dimensional model with S=2 and state 1 is ')
        disp('  absorbing, then type the following and run it')
        disp('         Opt.ks=[0;1]; find_tau2(P,FK,Opt);      ')
        disp('===================================')
    end
        
%% [5] Some Additional Information 
        LtkF=cell(S,S); 
            for i=1:S, for j=1:S
            LtkF{i,j}=kron(L{i,j}',F{i,j}); 
            end, end

        BarPsi_L=[]; Psi_LtkF=[]; 
            for i=1:S
            tmp1=[]; tmp2=[]; tmp3=[];
                for j=1:S, 
                    tmp1=[tmp1 P(j,i)*L{j,i}]; 
                    tmp2=[tmp2 P(i,j)*LtkF{i,j}];         
                end
            BarPsi_L=[BarPsi_L;tmp1];    
            Psi_LtkF=[Psi_LtkF;tmp2]; 
            end
        R_BarPsi_L=max(abs(eig(BarPsi_L))); 
        tau1=R_BarPsi_L;

        OtherOutput.Phi=Phi;
        OtherOutput.xi1=xi1;
        OtherOutput.xi2=xi2;
        OtherOutput.tau1=tau1;
        OtherOutput.Psi_LtkF=Psi_LtkF; 
    varargout{3}=OtherOutput;

    % Some results worth confirming.
    % (1) Psi_LtkF must contain a unit root.
    % (2) tau1=xi1/xi2; xi1=tau1/tau2; 
        %[tau1 xi1/xi2], [xi1 tau1/tau2]

